#include "ns_filter.h"

filterattrib::filterattrib(char buf[])
{
  filterid = 0;
  flowid = 0;
  chains = 0;
  imq_auto_redirect = false;
  dest_mark = false;
  from_localhost = false;
  to_localhost = false;
  ipt_return = false;
  use_tc_filter = true;  
  filter = u32;
  strcpy(dev, "");
  strcpy(match, buf);
  if (strstr(buf, "to localhost")) to_localhost = true;
  else if (strstr(buf, "from localhost")) from_localhost = true;
}

filterattrib::~filterattrib()
{
  char buf[LONG_BUF];

  if (from_localhost) { sprintf(buf, "iptables -t mangle -D OUTPUT %s -j %s ", ipt_match, ipt_mychain); system_ipt(buf); }
  if (to_localhost) { sprintf(buf, "iptables -t mangle -D INPUT %s -j %s ", ipt_match, ipt_mychain); system_ipt(buf); }
}
int filterattrib::store(char buf[])
{
  char option[35] = "";

  strcpy(option, awk(buf, 1));
  if (!strcmp(option, "class")) {
    strncpy(dev, awk(buf, 3), MAX_IFACE);
    dev[MAX_IFACE - 1] = '\0';
    if (strstr(markifaces, dev)) filter = fw; 
  }
  else if (!strcmp(option, "filterid")) filterid = atoi(awk(buf, 2));
  else if (!strcmp(option, "classid")) flowid = atoi(awk(buf, 2));
  else if (!strcmp(option, "do-not-shape")) {
    use_tc_filter = true;
    flowid = 1;
  } 
  else if (!strcmp(option, "virtual")) {
    use_tc_filter = false;
    ipt_return = true;    
  } 
  else if (!strcmp(option, "wrapper")) {
    use_tc_filter = true;
  } 
  else if (!strcmp(option, "imq-redirect")) {
    imq_auto_redirect = true;
  }
  else if (!strcmp(option, "iptables-return")) {
    ipt_return = true;
  }
  else return -1;
  
  return 1;
}

int filterattrib::prepare_tcfilter()
{
  char buf[LONG_BUF] = "";
  char buf2[LONG_BUF] = "";
  char addr[16];
  char mask[16];

  strcpy(tcfilter_add, "");
  strcpy(tcfilter_del, "");
  if ( !use_tc_filter ) return 0;
  if (filter == u32) {
    if (strstr(match, "proto")) {
      if (!strcmp(awk(strstr(match, "proto"), 2), "tcp")) strcat(buf, " match ip protocol 6 0xff ");
      else if (!strcmp(awk(strstr(match, "proto"), 2), "udp")) strcat(buf, " match ip protocol 17 0xff ");
      else if (!strcmp(awk(strstr(match, "proto"), 2), "icmp"))	strcat(buf, " match ip protocol 1 0xff ");
    }
    if (strstr(match, "srcip")) {
      if (split(addr, mask, awk(strstr(match, "srcip"), 2)) == -1) return -1;
      if (solid_ipmask(mask)) sprintf(buf2, " match ip src %s/%d ", addr, dotobit(mask));
      else { printf("%s: use solid mask with u32 filter\n", section); return -1; }
      strcat(buf, buf2);
    }
    if (strstr(match, "dstip")) {
      if (split(addr, mask, awk(strstr(match, "dstip"), 2)) == -1) return -1;
      if (solid_ipmask(mask)) sprintf(buf2, " match ip dst %s/%d ", addr, dotobit(mask));
      else { printf("%s: use solid mask with u32 filter\n", section); return -1; }
      strcat(buf, buf2);
    }
    if (strstr(match, "srcport")) {
      strcat(buf, " match ip sport ");
      strcat(buf, awk(strstr(match, "srcport"), 2));
      strcat(buf, " 0xffff ");
    }
    if (strstr(match, "dstport")) {
      strcat(buf, " match ip dport ");
      strcat(buf, awk(strstr(match, "dstport"), 2));
      strcat(buf, " 0xffff ");
    }
    sprintf(tcfilter_add, "tc filter add dev %s protocol ip parent 1:0 prio 10 handle 800::%x u32 %s flowid 1:%d", dev, filterid, buf, flowid);
    sprintf(tcfilter_del, "tc filter del dev %s protocol ip parent 1:0 prio 10 handle 800::%x u32", dev, filterid);
  } 
  else if (filter == fw) { 
    sprintf(tcfilter_add, "tc filter add dev %s protocol ip parent 1:0 prio 10 handle %d fw flowid 1:%d", dev, flowid, flowid);
    sprintf(tcfilter_del, "tc filter del dev %s protocol ip parent 1:0 prio 10 handle %d fw", dev, flowid);
  }
  return 1;
}

int filterattrib::sysinit_iptables()
{
  char buf[LONG_BUF] = "";

  if ( gen_ipt_filter (ipt_match, match) == -1 ) return -1;

  if (from_localhost) {
    strcat(ipt_match, " -o ");
    strcat(ipt_match, dev);
    strcat(ipt_match, " ");
    sprintf(buf, "iptables -t mangle -A OUTPUT %s -j %s ", ipt_match, ipt_mychain);
    system_ipt(buf);
  }
  if (to_localhost) {
    strcat(ipt_match, " -i ");
    strcat(ipt_match, dev);
    strcat(ipt_match, " ");
    sprintf(buf, "iptables -t mangle -A INPUT %s -j %s ", ipt_match, ipt_mychain);
    system_ipt(buf);
  }
  if (filter == fw) {
    chains++;
    sprintf(buf, "iptables -t mangle -A %s %s -j MARK --set-mark %d", ipt_mychain, ipt_match, flowid);
    system_ipt(buf);
  }
  if (imq_auto_redirect) {
    if (strstr(dev, "imq")) {
      chains++;
      sprintf(buf, "iptables -t mangle -A %s %s -j IMQ --todev %s", ipt_mychain, ipt_match, &(dev[3]));
      system_ipt(buf);
    }
  }
  chains++;  
  if ( ipt_return ) sprintf(buf, "iptables -t mangle -A %s %s -j RETURN", ipt_mychain, ipt_match);
  else sprintf(buf, "iptables -t mangle -A %s %s -j ACCEPT", ipt_mychain, ipt_match);
  system_ipt(buf);
  return 1;
}

unsigned int filterattrib::check(std::vector < char * > &iptables_summary)
{
  unsigned int n;
  char buf[LONG_BUF];

  for ( n = 0; n < chains; n++ ) {
	strncpy ( buf, iptables_summary[0] , LONG_BUF );
	delete [] iptables_summary[0];
    iptables_summary.erase(iptables_summary.begin());	
  }
  return (atol(awk(buf, 2)) * 8);
}
